#ifndef TREEITEM_H
#define TREEITEM_H

#include <QColor>
#include <QList>
#include <QVariant>

#include "noteobj.h"
#include "xmlobj.h"

class LinkableMapObj;
class BranchObj;
class BranchItem;


class VymModel;

class TreeItem:public XMLObj
{
public:
	enum Type {Undefined,MapCenter,Branch,Image};
	enum HideTmpMode {HideNone, HideExport};

    TreeItem(const QList<QVariant> &data, TreeItem *parent = 0);
    ~TreeItem();
	void init();

	virtual QString saveToDir (const QString &tmpdir,const QString &prefix, const QPointF& offset);


	// General housekeeping
	virtual void setModel (VymModel *m);
	virtual VymModel* getModel();

    virtual void appendChild (TreeItem *child);
	virtual void removeChild (int row);

    virtual TreeItem *child(int row);
    virtual int childCount() const;
    virtual int childNumber() const;
    virtual int columnCount() const;
	virtual int branchCount() const;
	virtual int imageCount() const;
	virtual int xlinkCount() const;

    virtual int row() const;
	virtual int column() const;
	virtual int depth() ;
    virtual TreeItem *parent();

	/*! Return number of item in parent by type, 
	    e.g. first branch has number 0           */
	virtual int childNum();				//! Return number of item in list of all children
	virtual int num();					//! Return number of item by type
	virtual int num (TreeItem *item);	//! Return number of item by type

	// Accessing data
    virtual QVariant data(int column) const;

protected:
	QColor headingColor;
public:	
	virtual void setHeading (const QString s);
	virtual QString getHeading() const;
	virtual std::string getHeadingStd() const;	//! convenience function used for debugging
	virtual void setHeadingColor(QColor color);	//! Set color of heading. In BranchItem overloaded to update QGraphicsView
	virtual QColor getHeadingColor();			//! Returns color of heading

protected:
	NoteObj note;
public:	
	virtual void setNote(const QString s);
	virtual QString getNote();
	virtual void setNoteObj(const NoteObj &, bool updateNoteEditor=true);
	virtual NoteObj getNoteObj();			
	virtual QString getNoteASCII(const QString &indent, const int &width); // returns note	(ASCII)
    virtual QString getNoteASCII();			// returns note	(ASCII)
    virtual QString getNoteOpenDoc();		// returns note	(OpenDoc)


protected:
	Type type;
public:	
	virtual void setType (const Type t);
	virtual Type getType ();
	virtual bool isBranchLikeType() const;
	virtual QString getTypeName ();

protected:
	QString objID;					//! id set during load/save currently used for xLinks
public:
	virtual void setID (const QString &s);
	virtual QString getID ();

	// Navigation and selection
	virtual TreeItem* getChildNum(const int &n);
	virtual BranchItem* getFirstBranch();
	virtual BranchItem* getLastBranch();
	virtual BranchItem* getBranchNum(const int &n);
	virtual BranchObj* getBranchObjNum(const int &n);
	virtual void setLastSelectedBranch();		//! Set myself as last selected in parent
	virtual void setLastSelectedBranch(int i);	//! Set last selected branch directly
	virtual TreeItem* getLastSelectedBranch();
	virtual TreeItem* findBySelectString (const QString &);	

	virtual void setHideTmp (HideTmpMode);
	virtual bool hasHiddenExportParent ();
	virtual void setHideInExport(bool);		// set export of object (and children)
	virtual bool hideInExport();
	virtual bool isHidden ();		

	
	// Relation to map objects in graphicsscene	// FIXME-4 should be obsolete
	virtual LinkableMapObj* getLMO();
	virtual void setLMO (LinkableMapObj*);

protected:
	VymModel *model;

    QList<TreeItem*> childItems;
    QList<QVariant> itemData;
    TreeItem *parentItem;
 
	LinkableMapObj *lmo;

	int branchOffset;
	int branchCounter;
	int lastSelectedBranchNum;

	bool hideExport;//! Hide this item in export
	bool hidden;	//! Hidden in export if true
};

#endif
